function T = MSTJ(X)
%MSTJ applica l'algoritmo di Jarnik Prim per trovare il mst
%dato un grafo euclideo

[n,~]=size(X);
%tabella che nelle righe contiene gli archi aggiunti nel grafo nella forma
%(predecessore, nodo, peso)
grafo=zeros(n-1,3);

%coda di priorit in forma di heap binario che contiene i nodi
%non ancora nel MST e la distanza minima dall'albero gi costruito
PQ=inf*ones(n-1,2);
PQ(:,1)=(2:n)';

%posizioni dei nodi all'interno dell'heap
pos=zeros(n,1);
pos(2:n)=1:n-1;

%inserisce gli archi che collegano il primo nodo agli altri e ne salva la
%posizione nella coda di priorit
vedges=vecnorm((X(2:n,:)-ones(n-1,1)*X(1,:)),2,2);
for k=1:n-1
    [PQ,pos]=decreasekey(PQ,n-1,pos,k,vedges(k));
end

%vettore dei predecessori
pred= ones(n,1);

%numero dei nodi ancora da inserire
n=n-1;

while ~isempty(PQ)
    
    %estrae il nodo pi vicino e lo aggiunge al MST con il relativo arco
    v=PQ(1,:);
    grafo(n,:)=[v(1),pred(v(1)),v(2)];
    [PQ,n,pos] = dequeue(PQ,n,pos);
   
    
    %vettore delle distanze dall'ultimo nodo agguinto dei nodi
    %non ancora nel MST    
    vedges=vecnorm((X((PQ(:,1)),:)-ones(n,1)*X(v(1),:)),2,2) ;
    
    %estrae solo le distanze che sono minori rispetto al MST con n-1 nodi
    vedges=vedges.*(PQ(:,2)>vedges);
    
    %aggiorna nella coda di priorit le distanze dei nodi, mantendendo la
    %struttura di heap, e il vettore dei predecessori
    for j=1:n
        if vedges(j)~=0
            pred(PQ(j,1))=v(1);
            [PQ,pos]=decreasekey(PQ,n,pos,j,vedges(j));
        end
    end
  
end

%costruisce il grafo di tipo graph a partire dalla tabella degli archi
Tab=table(grafo(:,1:2), grafo(:,3));
Tab.Properties.VariableNames={'EndNodes','Weight'};
T=graph(Tab); 
    
end