function grafo = MSTdivideetimperah(X,index, s,L,coor)

%dati:

%un vettore X di n punti in Rm
%un insieme di indici che identificano i nodi
%un nodo radice s nella forma s=[posizione di s nell'originale X, coordinate del nodo in Rn]
%la lunghezza del alto del cubo L
%il punto di coordinate minime fra i punti del quadrato che contiene di nodi

%calcola uno spanning tree di X con radice s usando un'euristica di tipo divide et
%impera che a ogni chiamata suddivide la regione in cubi di lunghezza del lato fissata
%e per ogni cubo calcola uno spanning tree al suo interno scegliendo un nodo qualunque
%come sua radice, per poi unire ogni albero generato al nodo s tramite la
%propria rispettiva radice


[n,m]=size(X);

if isempty(X)
    grafo=[];
end

if ~isempty(X)
    %tabella che nelle righe contiene gli archi aggiunti nel grafo nella forma
    %(predecessore, nodo, peso)
    grafo=zeros(n,3);
    
    k=floor(n^(1-1/m));
    
    %numero di cubi
    ncubes=(floor(k^(1/m)))^m;
    
    %numero di cubi che toccano un lato del cubo precedente
    sides=floor(k^(1/m));
    
    %array che indica a quale cubo appartengono i punti
    incube=zeros(n,1);
    
    %lunghezza dei lati dei cubi generati
    L=L/(sides);
    
    cube=zeros(m,1);
    aux=sides.^(0:m-1);
    
    
    for j=1:n
        %trova il cubo di appartenenza
        
        for dim=1:m
            cube(dim)=floor((X(j,dim)-coor(dim))/L);
        end
        
        %converte il cubo trovato in un intero
        
        poscube=(aux*cube)+1;
        
        %aggiunge a incube
        incube(j)=poscube;
    end
    
    
    t=1;
    cube=zeros(m,1);
    
    for j=1:ncubes
        
        %seleziona i nodi che appartengono al j-esimo cubo
        findcube=nonzeros((incube==j).*(1:n)');
        
        
        if ~isempty(findcube)
            lenf=length(findcube);
            
            %seleziona il nodo radice all'interno del cubo
            rad=X(findcube(1),:);
            dist=vecnorm(rad-s(2:end),2,2);
            ind=index(findcube(1));
            
            %aggiunge al grafo una arco che connette il cubo alla radice s
            grafo(t,:) =[ s(1),  ind, dist ];
            t=t+1;
            
            
            % aggiorna il punto di coordinate minime fra i punti del quadrato che contiene di nodi
            aux=j-1;
            for dim=m:-1:1
                cube(dim)=floor(aux/ sides^(dim-1) )*L; %divide indice del cubo per sapere in quale cubo n-1 dimensionale si trova
                aux=aux-floor(aux/sides^(dim-1))*sides^(dim-1);
            end
            
            
            %inserisce i nodi presenti nel cubo calcolando al suo interno uno
            %spanning tree
            if (lenf~=1) %altrimenti sono già stati aggiunti tutti i nodi
                grafo(t:t+lenf-2,:)=MSTdivideetimperah(X(findcube(2:end),:), index(findcube(2:end)),[ind,rad],L,coor+cube);
                t=t+lenf-1;
            end
        end
        
    end
end
end


